@@ -12,7 +12,7 @@
 # This program is free software; you can redistribute it and/or
 # modify it under the same terms as Perl itself.
 #
-# $Id: TestSuite.pm,v 1.12 2002/02/24 14:22:23 book Exp $
+# $Id: TestSuite.pm,v 1.17 2002/02/26 20:40:47 book Exp $
 #
 package Games::Golf::TestSuite;
 
@@ -21,22 +21,23 @@ use strict;
 # Modules we rely upon.
 use Carp;
 use File::Basename;
-use vars qw/ $VERSION /;
+use IO::Select;
+use IPC::Open3;
+use POSIX ":sys_wait_h";
 
 # Variables of the module.
 local $^W = 1;    # use warnings for perl < 5.6
-$VERSION  = '0.08';
 
 =head1 NAME
 
-Game::Golf::TestSuite - An object that can run a suite of tests.
+Games::Golf::TestSuite - An object that can run a suite of tests.
 
 =head1 SYNOPSIS
 
-  use Game::Golf;
+  use Games::Golf;
 
   # "hole" is the file holding the test suite
-  my $test = new Game::Golf::TestSuite( "hole" );
+  my $test = new Games::Golf::TestSuite( "hole" );
 
   # $entry is a Games::Golf::Entry object
   $test->check( $entry );
@@ -63,7 +64,7 @@ A test suite is a perl script written as follow:
  $out  = 'the script expected output';
  $err  = '';    # don't expect anything on STDERR
  $exit = undef; # don't care for exit code
- $test->IOEE( $in, $out, $err, $exit );
+ $test->ioee( $in, $out, $err, $exit );
 
  # use other Games::Golf::TestSuite method
  # to create as many subtests as needed
@@ -91,7 +92,7 @@ sub new {
         local \$^W;
         my \$test = shift; # this is the Games::Golf::TestSuite
         # create a "sandbox", so as to avoid variable conflicts
-        package Games::Golf::Test::Sandbox;
+        package Games::Golf::TestSuite::Sandbox;
         # insert the test code
         $testsuite;
         # return the results
@@ -116,15 +117,14 @@ This method simply loops on C<@entries> with check() method.
 sub run {
     my $self    = shift;
     my @entries = @_;
-    for (@entries) {
-        $_->result( $self->check($_) );
-    }
+    for my $entry (@entries) { $self->check( $entry ) }
 }
 
 =item $test->check( $entry );
 
 Run the testsuite on a single Games::Golf::Entry object, update the
-object and return the results of the test.
+object and return the results of the test. The C<result> attribute
+of the C<Games::Golf::Entry> object is updated.
 
 =cut
 
@@ -182,6 +182,7 @@ in their hole test scripts.
 =item $test->compile;
 
 Does the player script at least compile?
+This does count as a test in the testsuite.
 
 =cut
 
@@ -192,6 +193,7 @@ sub compile {
     # !!FIXME!! does in work under Win32?
     # couldn't we fork a child and read its STDERR
     # directly?
+    #  use POSIX qw(tmpnam)
     my $result = qx($^X -c $file 2>err.tmp);
 
     open ERR, "<err.tmp" or return $!;
@@ -213,16 +215,116 @@ sub compile {
     }
 }
 
-=item $test->IOEE( $in, $out, $err, $exit );
+=item $test->ioee( $in, $out, $err, $exit, time => $time, stdout => $limit, stderr => $limit );
 
-Given C<$in>, the script should output C<$out> on STDOUT, C<$err> on STDERR
-and return the exit code C<$exit>. If you don't care about the exit code,
-don't give it (or pass C<undef>).
+Given C<$in>, the script should output C<$out> on STDOUT, C<$err> on
+STDERR and return the exit code C<$exit>. If you don't care about any
+of the tree outputs (stdout, stderr or exit code), just pass C<undef>.
+
+You can pass further three limits to tune the test:
+
+=over 4
+
+=item o
+
+A time limit for each test (default 30 seconds).
+
+=item o
+
+A data limit for stdout (default 1 megabyte).
+
+=item o
+
+A data limit for stderr (default 1 megabyte).
+
+=back
+
+B<WARNING!> This sub is a minimal workaround, and may deadlock on big
+input sets, since we do not check via select if we can write.
 
 =cut
 
-# Jonathan take responsibility of this one.
-sub IOEE { }
+sub ioee {
+    my ( $self, $in, $out, $err, $exit ) = splice @_, 0, 5;
+
+    use constant SELECT_TIMEOUT => 0.05;
+
+    my $file = $self->{file} . ".pl";
+
+    # Restrictions to apply
+    my %restrict = 
+      ( time   => 30,		# 30 Seconds
+	stdout => 1000000,	# 1  Megabyte
+	stderr => 1000000,	# 1  Megabyte
+	@_
+      );
+
+    croak "Invalid parameters passed!"
+      unless (scalar keys %restrict == 3);
+
+    # Storage for data read
+    my ($stdout, $stderr) = ("", "");
+
+
+    # Use exceptions
+    eval {
+        # Start code under test
+	local (*IN, *OUT, *ERR);
+	my $pid  = open3(\*IN, \*OUT, \*ERR, "$^X $file");
+
+	# Use select to monitor filehandles
+	my $select = IO::Select->new;
+	$select->add(\*OUT, \*ERR); # do not check \*IN.
+
+	# Do writing.
+	print IN $in;
+	close IN;
+
+	# Do reading
+	while ( my @ready = $select->can_read(SELECT_TIMEOUT) ) {
+	    foreach my $fh ( @ready ) {
+		# from STDOUT
+		if ($fh == \*OUT) {
+		    $stdout .= <OUT>;
+		}
+		# from STDERR
+		elsif ($fh == \*ERR) {
+		    $stderr .= <ERR>;
+		}
+		$select->remove($fh) if eof($fh);
+	    }
+	}
+	close OUT;
+	close ERR;
+    };
+
+    # Handle exceptions (TODO)
+    #if ($@) {
+    #	die $@;
+    #}
+
+    # one more test.
+    $self->{result}[0]++;
+    my $success = 1;
+    my $msg     = "";
+
+    if ( $stdout ne $out ) {
+	# Ooops, wrong output.
+	$success = 0;
+        $msg = "Oops, wrong output.\nExpected:\n--\n".$out."--\nGot:\n--\n".$stdout."--";
+
+    } 
+
+    if ( $stderr ne $err ) {
+	# Ooops, wrong stderr.
+	$success = 0;
+        $msg = "Oops, wrong stderr.\nExpected:\n--\n".$err."--\nGot:\n--\n".$stderr."--";
+    } 
+
+    # Test is ok.
+    $self->{result}[1] += $success;
+    push @{ $self->{result} }, $msg;
+}
 
 =head2 Available tests for testing subs
 
@@ -239,6 +341,9 @@ Create a subroutine named after the hole in the sandbox.
 It can then be used through C<$test-E<gt>sub( ... )> or by calling it
 directly by its name (the hole name).
 
+This does count as a test in the testsuite. The sub should at least be
+valid Perl, shouldn't it?
+
 =cut
 
 sub makesub {
@@ -253,10 +358,10 @@ sub makesub {
     push @{ $self->{result} }, $@;
 
     # finally store the coderef (might be undef)
-    no strict 'refs';
     $self->{sub} = $sub;
 
     # does nothing if $sub is undef
+    no strict 'refs';
     local $^W;    # don't warn about redefined subs
     *{"Games::Golf::TestSuite::Sandbox::$self->{hole}"} = $sub;
 }
@@ -422,7 +527,7 @@ and/or modified under the same terms as Perl itself.
 
 =head1 SEE ALSO
 
-L<perl>, L<Game::Golf>.
+L<perl>, L<Games::Golf>, L<Games::Golf::Entry>.
 
 =cut
 
